# flake8: noqa
"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.okta_object import OktaObject
from okta.okta_collection import OktaCollection
from okta.models import log_actor\
    as log_actor
from okta.models import log_authentication_context\
    as log_authentication_context
from okta.models import log_client\
    as log_client
from okta.models import log_debug_context\
    as log_debug_context
from okta.models import log_outcome\
    as log_outcome
from okta.models import log_request\
    as log_request
from okta.models import log_security_context\
    as log_security_context
from okta.models import log_severity\
    as log_severity
from okta.models import log_target\
    as log_target
from okta.models import log_transaction\
    as log_transaction


class LogEvent(
    OktaObject
):
    """
    A class for LogEvent objects.
    """

    def __init__(self, config=None):
        super().__init__(config)
        if config:
            if "actor" in config:
                if isinstance(config["actor"],
                              log_actor.LogActor):
                    self.actor = config["actor"]
                elif config["actor"] is not None:
                    self.actor = log_actor.LogActor(
                        config["actor"]
                    )
                else:
                    self.actor = None
            else:
                self.actor = None
            if "authenticationContext" in config:
                if isinstance(config["authenticationContext"],
                              log_authentication_context.LogAuthenticationContext):
                    self.authentication_context = config["authenticationContext"]
                elif config["authenticationContext"] is not None:
                    self.authentication_context = log_authentication_context.LogAuthenticationContext(
                        config["authenticationContext"]
                    )
                else:
                    self.authentication_context = None
            else:
                self.authentication_context = None
            if "client" in config:
                if isinstance(config["client"],
                              log_client.LogClient):
                    self.client = config["client"]
                elif config["client"] is not None:
                    self.client = log_client.LogClient(
                        config["client"]
                    )
                else:
                    self.client = None
            else:
                self.client = None
            if "debugContext" in config:
                if isinstance(config["debugContext"],
                              log_debug_context.LogDebugContext):
                    self.debug_context = config["debugContext"]
                elif config["debugContext"] is not None:
                    self.debug_context = log_debug_context.LogDebugContext(
                        config["debugContext"]
                    )
                else:
                    self.debug_context = None
            else:
                self.debug_context = None
            self.display_message = config["displayMessage"]\
                if "displayMessage" in config else None
            self.event_type = config["eventType"]\
                if "eventType" in config else None
            self.legacy_event_type = config["legacyEventType"]\
                if "legacyEventType" in config else None
            if "outcome" in config:
                if isinstance(config["outcome"],
                              log_outcome.LogOutcome):
                    self.outcome = config["outcome"]
                elif config["outcome"] is not None:
                    self.outcome = log_outcome.LogOutcome(
                        config["outcome"]
                    )
                else:
                    self.outcome = None
            else:
                self.outcome = None
            self.published = config["published"]\
                if "published" in config else None
            if "request" in config:
                if isinstance(config["request"],
                              log_request.LogRequest):
                    self.request = config["request"]
                elif config["request"] is not None:
                    self.request = log_request.LogRequest(
                        config["request"]
                    )
                else:
                    self.request = None
            else:
                self.request = None
            if "securityContext" in config:
                if isinstance(config["securityContext"],
                              log_security_context.LogSecurityContext):
                    self.security_context = config["securityContext"]
                elif config["securityContext"] is not None:
                    self.security_context = log_security_context.LogSecurityContext(
                        config["securityContext"]
                    )
                else:
                    self.security_context = None
            else:
                self.security_context = None
            if "severity" in config:
                if isinstance(config["severity"],
                              log_severity.LogSeverity):
                    self.severity = config["severity"]
                elif config["severity"] is not None:
                    self.severity = log_severity.LogSeverity(
                        config["severity"].upper()
                    )
                else:
                    self.severity = None
            else:
                self.severity = None
            self.target = OktaCollection.form_list(
                config["target"] if "target"\
                    in config else [],
                log_target.LogTarget
            )
            if "transaction" in config:
                if isinstance(config["transaction"],
                              log_transaction.LogTransaction):
                    self.transaction = config["transaction"]
                elif config["transaction"] is not None:
                    self.transaction = log_transaction.LogTransaction(
                        config["transaction"]
                    )
                else:
                    self.transaction = None
            else:
                self.transaction = None
            self.uuid = config["uuid"]\
                if "uuid" in config else None
            self.version = config["version"]\
                if "version" in config else None
        else:
            self.actor = None
            self.authentication_context = None
            self.client = None
            self.debug_context = None
            self.display_message = None
            self.event_type = None
            self.legacy_event_type = None
            self.outcome = None
            self.published = None
            self.request = None
            self.security_context = None
            self.severity = None
            self.target = []
            self.transaction = None
            self.uuid = None
            self.version = None

    def request_format(self):
        parent_req_format = super().request_format()
        current_obj_format = {
            "actor": self.actor,
            "authenticationContext": self.authentication_context,
            "client": self.client,
            "debugContext": self.debug_context,
            "displayMessage": self.display_message,
            "eventType": self.event_type,
            "legacyEventType": self.legacy_event_type,
            "outcome": self.outcome,
            "published": self.published,
            "request": self.request,
            "securityContext": self.security_context,
            "severity": self.severity,
            "target": self.target,
            "transaction": self.transaction,
            "uuid": self.uuid,
            "version": self.version
        }
        parent_req_format.update(current_obj_format)
        return parent_req_format



from okta.oauth import OAuth

"""
Testing OAuth.clear_access_token
"""
class WasCalled:
    def __init__(self):
        self.value = False

cache_delete_was_called = WasCalled
headers_pop_was_called = WasCalled

class mockCache:
    def delete(token):
        cache_delete_was_called.value = True

class mockHeaders:
    def pop(header, ignored):
        headers_pop_was_called.value = True

class mockRequestExecutor:
    def __init__(self, cache):
        self._cache = cache
        self._default_headers = mockHeaders

def test_oauth_clear_access_token():
    _mockRequestExecutor = mockRequestExecutor(mockCache)
    oauth = OAuth(_mockRequestExecutor, {})
    oauth.clear_access_token()
    assert cache_delete_was_called.value
    assert headers_pop_was_called.value




import json
import os
import time
import uuid

from ast import literal_eval
from Cryptodome.PublicKey import RSA
from jwcrypto.jwk import JWK, InvalidJWKType
from jwt import encode as jwt_encode


class JWT():
    """
    This class creates a JWT from the Okta Client configuration.
    """

    OAUTH_ENDPOINT = "/oauth2/v1/token"
    HASH_ALGORITHM = "RS256"
    PEM_FORMAT = "PKCS1"
    ONE_HOUR = 1 * 60 * 60
    JWT_OPTIONS = {
        'verify_signature': True,
        'verify_aud': True,
        'verify_iat': True,
        'verify_exp': True,
        'verify_nbf': True,
        'verify_iss': True,
        'verify_sub': True,
        'verify_jti': True,
        'verify_at_hash': True,
        'require_aud': True,
        'require_iat': False,
        'require_exp': True,
        'require_nbf': False,
        'require_iss': True,
        'require_sub': True,
        'require_jti': True,
    }

    def __init__(self, private_key):
        pass

    @staticmethod
    def get_PEM_JWK(private_key):
        """
        This class gets the PEM and JWK representation of the private key
        from the Okta Client configuration.

        Args:
            private_key (str or dict): Either a dict representing a JWK, str
            representing the JWK, filepath to PEM, or PEM string

        Returns:
            RSAKey, JWK: Tuple of the generated PEM, and JWK
        """
        # Start off with both as None
        my_jwk = None
        my_pem = None

        # check if JWK
        # String representation of dictionary or dict
        if ((isinstance(private_key, str) and private_key.startswith("{")) or
                isinstance(private_key, dict)):
            # if string repr, convert to dict object
            if isinstance(private_key, str):
                private_key = literal_eval(private_key)
            # remove whitespace from key vaules
            private_key = {k: ''.join(private_key[k].split()) for k in private_key}
            # ensure private_key is JSON formatted
            try:
                json.loads(private_key)
            except TypeError:
                private_key = json.dumps(private_key)
            try:
                my_jwk = JWK.from_json(private_key)
            except InvalidJWKType:
                raise ValueError(
                    "JWK given is of the wrong type")
        else:  # it's a PEM
            # check for filepath or explicit private key
            if isinstance(private_key, (str, bytes, os.PathLike)) and os.path.exists(private_key):
                # open file if exists and read
                pem_file = open(private_key, 'r')
                private_key = pem_file.read()
                pem_file.close()
            # remove leading whitespaces from each line
            my_pem = '\n'.join([line.strip() for line in private_key.splitlines()])
            my_pem = bytes(my_pem, 'ascii')
            try:
                my_jwk = JWK.from_pem(my_pem)
            except ValueError:
                raise ValueError(
                    "RSA Private Key given is of the wrong type")

        my_pem = my_jwk.export_to_pem(private_key=True, password=None)
        my_pem = RSA.import_key(my_pem)

        return (my_pem, my_jwk)

    @staticmethod
    def create_token(org_url, client_id, private_key, kid=None):
        """
        Create a JSON Web Token using Oauth details from the Okta Client
        config.

        Args:
            org_url (str): Okta Organization URL
            client_id (str): Client ID
            private_key (str or dict): Private Key representation
            kid (str): Key ID

        Returns:
            str: Generated JWT
        """
        # Generate PEM and JWK
        my_pem, _ = JWT.get_PEM_JWK(private_key)
        # Get current time and expiry time for token
        issued_time = int(time.time())
        expiry_time = issued_time + JWT.ONE_HOUR
        # generate unique JWT ID
        generated_JWT_ID = str(uuid.uuid4())

        # Create claims for token and create token
        claims = {
            'sub': client_id,
            'iat': issued_time,
            'exp': expiry_time,
            'iss': client_id,
            'aud': org_url + JWT.OAUTH_ENDPOINT,
            'jti': generated_JWT_ID
        }

        # Add additional headers
        headers = {}

        # # Check if kid was supplied
        if kid:
            headers["kid"] = kid
        elif isinstance(private_key, dict) and "kid" in private_key:
            headers["kid"] = private_key["kid"]
        elif isinstance(private_key, str):
            try:
                private_key_dict = json.loads(private_key)
                if "kid" in private_key_dict:
                    headers["kid"] = private_key_dict["kid"]
            except json.JSONDecodeError:
                if "kid" in headers:
                    del headers["kid"]

        token = jwt_encode(claims, my_pem.export_key(), JWT.HASH_ALGORITHM, headers)
        return token



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from urllib.parse import urlencode
from okta.models.brand\
    import Brand
from okta.models.email_template\
    import EmailTemplate
from okta.models.email_template_customization\
    import EmailTemplateCustomization
from okta.models.email_template_content\
    import EmailTemplateContent
from okta.models.theme_response\
    import ThemeResponse
from okta.models.image_upload_response\
    import ImageUploadResponse
from okta.utils import format_url
from okta.api_client import APIClient


class BrandClient(APIClient):
    """
    A Client object for the Brand resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_brands(
            self,
            keep_empty_params=False
    ):
        """
        List all the brands in your org.
        Args:
        Returns:
            list: Collection of Brand instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Brand)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(Brand(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_brand(
            self, brandId,
            keep_empty_params=False
    ):
        """
        Fetches a brand by `brandId`
        Args:
            brand_id {str}
        Returns:
            Brand
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Brand)

        if error:
            return (None, response, error)

        try:
            result = Brand(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_brand(
            self, brandId, brand,
            keep_empty_params=False
    ):
        """
        Updates a brand by `brandId`
        Args:
            brand_id {str}
            {brand}
        Returns:
            Brand
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}
            """)

        if isinstance(brand, dict):
            body = brand
        else:
            body = brand.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Brand)

        if error:
            return (None, response, error)

        try:
            result = Brand(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_email_templates(
            self, brandId, query_params={},
            keep_empty_params=False
    ):
        """
        List email templates in your organization with paginati
        on.
        Args:
            brand_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.after] {str}
            [query_params.limit] {str}
        Returns:
            list: Collection of EmailTemplate instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplate)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(EmailTemplate(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_email_template(
            self, brandId, templateName,
            keep_empty_params=False
    ):
        """
        Fetch an email template by templateName
        Args:
            brand_id {str}
            template_name {str}
        Returns:
            EmailTemplate
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplate)

        if error:
            return (None, response, error)

        try:
            result = EmailTemplate(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_email_template_customizations(
            self, brandId, templateName,
            keep_empty_params=False
    ):
        """
        Delete all customizations for an email template. Also k
        nown as “Reset to Default”.
        Args:
            brand_id {str}
            template_name {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/customizations
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def list_email_template_customizations(
            self, brandId, templateName,
            keep_empty_params=False
    ):
        """
        List all email customizations for an email template
        Args:
            brand_id {str}
            template_name {str}
        Returns:
            list: Collection of EmailTemplateCustomization instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/customizations
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplateCustomization)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(EmailTemplateCustomization(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_email_template_customization(
            self, brandId, templateName, email_template_customization_request,
            keep_empty_params=False
    ):
        """
        Create an email customization
        Args:
            brand_id {str}
            template_name {str}
            {email_template_customization_request}
        Returns:
            EmailTemplateCustomization
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/customizations
            """)

        if isinstance(email_template_customization_request, dict):
            body = email_template_customization_request
        else:
            body = email_template_customization_request.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplateCustomization)

        if error:
            return (None, response, error)

        try:
            result = EmailTemplateCustomization(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_email_template_customization(
            self, brandId, templateName, customizationId,
            keep_empty_params=False
    ):
        """
        Delete an email customization
        Args:
            brand_id {str}
            template_name {str}
            customization_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/customizations/{customizationId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_email_template_customization(
            self, brandId, templateName, customizationId,
            keep_empty_params=False
    ):
        """
        Fetch an email customization by id.
        Args:
            brand_id {str}
            template_name {str}
            customization_id {str}
        Returns:
            EmailTemplateCustomization
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/customizations/{customizationId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplateCustomization)

        if error:
            return (None, response, error)

        try:
            result = EmailTemplateCustomization(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_email_template_customization(
            self, brandId, templateName, customizationId, email_template_customization_request,
            keep_empty_params=False
    ):
        """
        Update an email customization
        Args:
            brand_id {str}
            template_name {str}
            customization_id {str}
            {email_template_customization_request}
        Returns:
            EmailTemplateCustomization
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/customizations/{customizationId}
            """)

        if isinstance(email_template_customization_request, dict):
            body = email_template_customization_request
        else:
            body = email_template_customization_request.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplateCustomization)

        if error:
            return (None, response, error)

        try:
            result = EmailTemplateCustomization(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_email_template_customization_preview(
            self, brandId, templateName, customizationId,
            keep_empty_params=False
    ):
        """
        Get a preview of an email template customization.
        Args:
            brand_id {str}
            template_name {str}
            customization_id {str}
        Returns:
            EmailTemplateContent
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/customizations/{customizationId}
                /preview
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplateContent)

        if error:
            return (None, response, error)

        try:
            result = EmailTemplateContent(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_email_template_default_content(
            self, brandId, templateName,
            keep_empty_params=False
    ):
        """
        Fetch the default content for an email template.
        Args:
            brand_id {str}
            template_name {str}
        Returns:
            EmailTemplateContent
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/default-content
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplateContent)

        if error:
            return (None, response, error)

        try:
            result = EmailTemplateContent(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_email_template_default_content_preview(
            self, brandId, templateName,
            keep_empty_params=False
    ):
        """
        Fetch a preview of an email template's default content
        by populating velocity references with the current user
        's environment.
        Args:
            brand_id {str}
            template_name {str}
        Returns:
            EmailTemplateContent
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/default-content/preview
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, EmailTemplateContent)

        if error:
            return (None, response, error)

        try:
            result = EmailTemplateContent(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def send_test_email(
            self, brandId, templateName, email_template_test_request,
            keep_empty_params=False
    ):
        """
        Send a test email to the current users primary and seco
        ndary email addresses. The email content is selected ba
        sed on the following priority: An email customization s
        pecifically for the users locale. The default language
        of email customizations. The email templates default co
        ntent.
        Args:
            brand_id {str}
            template_name {str}
            {email_template_test_request}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/templates/email
                /{templateName}/test
            """)

        if isinstance(email_template_test_request, dict):
            body = email_template_test_request
        else:
            body = email_template_test_request.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def list_brand_themes(
            self, brandId,
            keep_empty_params=False
    ):
        """
        List all the themes in your brand
        Args:
            brand_id {str}
        Returns:
            list: Collection of ThemeResponse instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ThemeResponse)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(ThemeResponse(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_brand_theme(
            self, brandId, themeId,
            keep_empty_params=False
    ):
        """
        Fetches a theme for a brand
        Args:
            brand_id {str}
            theme_id {str}
        Returns:
            ThemeResponse
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ThemeResponse)

        if error:
            return (None, response, error)

        try:
            result = ThemeResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_brand_theme(
            self, brandId, themeId, theme,
            keep_empty_params=False
    ):
        """
        Updates a theme for a brand
        Args:
            brand_id {str}
            theme_id {str}
            {theme}
        Returns:
            ThemeResponse
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}
            """)

        if isinstance(theme, dict):
            body = theme
        else:
            body = theme.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ThemeResponse)

        if error:
            return (None, response, error)

        try:
            result = ThemeResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_brand_theme_background_image(
            self, brandId, themeId,
            keep_empty_params=False
    ):
        """
        Deletes a Theme background image
        Args:
            brand_id {str}
            theme_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}
                /background-image
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def upload_brand_theme_background_image(
            self, brandId, themeId, file,
            keep_empty_params=False
    ):
        """
        Updates the background image for your Theme
        Args:
            brand_id {str}
            theme_id {str}
        Returns:
            ImageUploadResponse
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}
                /background-image
            """)

        body = {}
        headers = {}
        form = {
            "file": file,
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ImageUploadResponse)

        if error:
            return (None, response, error)

        try:
            result = ImageUploadResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_brand_theme_favicon(
            self, brandId, themeId,
            keep_empty_params=False
    ):
        """
        Deletes a Theme favicon. The org then uses the Okta def
        ault favicon.
        Args:
            brand_id {str}
            theme_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}/favicon
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def upload_brand_theme_favicon(
            self, brandId, themeId, file,
            keep_empty_params=False
    ):
        """
        Updates the favicon for your theme
        Args:
            brand_id {str}
            theme_id {str}
        Returns:
            ImageUploadResponse
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}/favicon
            """)

        body = {}
        headers = {}
        form = {
            "file": file,
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ImageUploadResponse)

        if error:
            return (None, response, error)

        try:
            result = ImageUploadResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_brand_theme_logo(
            self, brandId, themeId,
            keep_empty_params=False
    ):
        """
        Deletes a Theme logo. The org then uses the Okta defaul
        t logo.
        Args:
            brand_id {str}
            theme_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}/logo
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def upload_brand_theme_logo(
            self, brandId, themeId, file,
            keep_empty_params=False
    ):
        """
        Updates the logo for your Theme
        Args:
            brand_id {str}
            theme_id {str}
        Returns:
            ImageUploadResponse
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}/logo
            """)

        body = {}
        headers = {}
        form = {
            "file": file,
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ImageUploadResponse)

        if error:
            return (None, response, error)

        try:
            result = ImageUploadResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)
